/***************************************************************
  Klondike for Gameboy
****************************************************************
  Written by Harold Toler (herg@bigfoot.com)
  August 18, 2000
****************************************************************
  Please email me before using this code in a project.  I'd
  not only like to know who's using it and for what,  but I'd
  also like to see your finished product.
  
  I'm sure the code isn't the best, but it works.  If you
  notice something I should do differently, let me know.
***************************************************************/

#include <rand.h>
#include <types.h>
#include <stdio.h>
#include <gb\gb.h>
#include <gb\cgb.h>

#include "backgrnd.h"
#include "backgrnd.c"  //data for background, generated by GBTD

#include "cardback.c"  //data for cards, generated by GBTD
#include "hole.c"
#include "trapcard.c"
#include "suits.c"
#include "middles.c"
#include "numbers.c"

#include "sprites.c"  //data for pointer and dot, generated by GBTD

#include "maps.c"     //maps for the cards and background

#include "start.c"

#include "sign.h"
#include "sign.c"
#include "signmap.c"

#include "settings.c"
#include "settings_map.c"

#include "color.c"     //color palettes

#define SPADES   0
#define CLUBS    1
#define HEARTS   2
#define DIAMONDS 3

#define HOLE 1
#define BACK 2
#define TRAP 3

#define SUIT_MASK       3
#define RANK_MASK       60
#define VISIBILITY_MASK 128

/***************************************************************
  Function prototypes
***************************************************************/
void title_screen(void);
void settings_screen(void);
void init_graphics(void);
void draw_background(void);
void draw_card(UBYTE x,UBYTE y,UBYTE rank,UBYTE suit);
void draw_partial_card(UBYTE x,UBYTE y,UBYTE rank,UBYTE suit);
void refresh_cards(void);
void place_pointer(UBYTE x,UBYTE y);
void place_dot(UBYTE x,UBYTE y);
void process_click(void);
UBYTE make_visible(UBYTE card);
UBYTE get_suit(UBYTE card);
UBYTE get_rank(UBYTE card);
UBYTE get_visibility(UBYTE card);
void deal_cards(void);
void move_deck_discard(void);
void move_discard_deck(void);
void move_deck_found(UBYTE pile);
void move_discard_goal(UBYTE pile);
void move_discard_found(UBYTE pile);
void move_found_goal(UBYTE found_pile, UBYTE goal_pile);
void move_found_found(UBYTE cards_to_move, UBYTE source_pile, UBYTE dest_pile);
void check_joypad(void);
void pause_game(void);
void win_animation(void);

/***************************************************************
  Global variables
***************************************************************/
UBYTE pointer_x,pointer_y;
UBYTE num_to_draw;
UBYTE discard_active;
UBYTE foundation_active;

UBYTE deck[52];
UBYTE foundation[7][19];
UBYTE discard[24];
UBYTE goal[4][13];

UBYTE deckindex;
UBYTE discardindex;
UBYTE goalindex[4];
UBYTE foundindex[7];

unsigned char blank_tile[] = {0};

UWORD seed;

/***************************************************************
  Data used to draw the sprites
***************************************************************/
unsigned char sprites_tiles[] =
{
  0,1,2,3,4,5,6,7,8,9,10,11,12
};

/***************************************************************
  Draws the title screen
***************************************************************/
void title_screen(void)
{
  UWORD i,j,k,x,y,card,offset;
  
  SHOW_BKG;
  SHOW_SPRITES;

  offset=150;
    
  k=0;
  while(!((joypad())&(J_START)))
  {
    k++;
    delay(3);
    if(k<500)
    {
      do{card=rand();}while(card>51);
      do{x=rand();}while(x>20);
      do{y=rand();}while(y>18);
      if(_cpu == CGB_TYPE)
      { 
        VBK_REG = 1;
        if(card&2) set_bkg_tiles(x,y,2,3,literedcard_color_map);
        else set_bkg_tiles(x,y,2,3,liteblackcard_color_map);
        VBK_REG = 0; 
      }
      set_bkg_tiles(x,y,2,3,card_maps[card+3]);
    }
    else if(k==700)
    {
      if(_cpu == CGB_TYPE)
      { 
        VBK_REG = 1;
        for(i=2;i<=17;i++) 
          for(j=2;j<=6;j++)
            set_bkg_tiles(i,j,1,1,sign_color_map);
        VBK_REG = 0; 
      }
      set_bkg_tiles(2,2,16,5,sign_tiles);
    }
    else if(k>1000 && offset!=0)
    {
      for(i=0;i<10;i++) move_sprite(i+3,49+i*8,140+(int)(offset/6));
      if(offset!=0) offset--;
    }
    else if(offset==0)
    {
      if(k&128) for(i=0;i<10;i++) move_sprite(i+3,49+i*8,210);
      else for(i=0;i<10;i++) move_sprite(i+3,49+i*8,140);
    }
  }
  waitpadup();
  for(i=0;i<10;i++)
    move_sprite(i+3,40+i*8,190);  //find better way to clear sprites
  seed = DIV_REG;

  DISPLAY_OFF;
  set_win_data(78,16,settings_data);
  move_win(7,144);
  DISPLAY_ON;

  pointer_x=20;
  pointer_y=20;
}

/***************************************************************
  Draws the settings screen
***************************************************************/
void settings_screen(void)
{
  UBYTE i,j;
  
  if(_cpu == CGB_TYPE)
  { 
    VBK_REG = 1;
    for(i=0;i<20;i++)
      for(j=0;j<18;j++)
        set_win_tiles(i,j,1,1,sign_color_map);
    VBK_REG = 0; 
  }
  set_win_tiles(0,0,20,4,settings_tiles);
  if(num_to_draw==3)
  {
    set_win_tiles(3,1,1,1,&settings_tiles[23]);
    set_win_tiles(3,2,1,1,blank_tile);
  }
  else
  {
    set_win_tiles(3,1,1,1,blank_tile);
    set_win_tiles(3,2,1,1,&settings_tiles[23]);
  }
  
  HIDE_SPRITES;
  SHOW_WIN;

  for(i=0;i<32;i++)
  {
    scroll_win(0,-1);
    delay(7);
  }
  
  while(!((joypad()&J_START) || (joypad()&J_A) || (joypad()&J_B)))
  {
    if((joypad()&J_UP)||(joypad()&J_DOWN)||(joypad()&J_SELECT))
    {
      waitpadup();
      if(num_to_draw==3)
      {
        num_to_draw=1;
        set_win_tiles(3,1,1,1,blank_tile);
        set_win_tiles(3,2,1,1,&settings_tiles[23]);
      }
      else
      {
        num_to_draw=3;
        set_win_tiles(3,1,1,1,&settings_tiles[23]);
        set_win_tiles(3,2,1,1,blank_tile);
      }
    }
  }
  waitpadup();
  seed = seed<<8 | (UWORD)DIV_REG;

  initrand(seed);

  for(i=0;i<32;i++)
  {
    scroll_win(0,1);
    delay(7);
  }
  
  SHOW_SPRITES;
  HIDE_WIN;
}

/***************************************************************
  Sets up all the graphics
***************************************************************/
void init_graphics(void)
{
  UBYTE i;

  DISPLAY_OFF;
  set_bkg_data(1,6,cardback_data);
  set_bkg_data(7,6,hole_data);
  set_bkg_data(13,2,trapcard_data);
  set_bkg_data(15,8,suits_data);
  set_bkg_data(23,2,middles_data);
  set_bkg_data(25,52,numbers_data);
  set_bkg_data(77,1,backgrnd_data);
  set_bkg_data(78,48,sign_data);
  set_sprite_data(0,3,sprites_data);
  set_sprite_data(3,10,start_data);

  for(i=0;i<13;i++)
    set_sprite_tile(i,sprites_tiles[i]);
  
  if(_cpu == CGB_TYPE)
  { 
    set_bkg_palette(0,7,&backgroundpalette[0]); 
    set_sprite_palette(0,2,&spritepalette[0]);
    set_sprite_prop(0,0);
    set_sprite_prop(1,0);
    set_sprite_prop(2,1);
    for(i=3;i<=12;i++) set_sprite_prop(i,0);
  } 

  DISPLAY_ON;
}

/***************************************************************
  Draws the background
***************************************************************/
void draw_background(void)
{
  UBYTE i,j;
  
  if(_cpu == CGB_TYPE)
  { 
    VBK_REG = 1;
    set_bkg_tiles(0,0,20,18,background_color_map);
    VBK_REG = 0; 
  }
  set_bkg_tiles(0,0,20,18,background_tiles);
}

/***************************************************************
  Draw a card on the screen
***************************************************************/
void draw_card(UBYTE x,UBYTE y,UBYTE rank,UBYTE suit)
{
  UBYTE index;
  
  index=(rank<<2)+suit-1;

  if(_cpu == CGB_TYPE)
  { 
    VBK_REG = 1;
    if(rank==0 && suit==BACK) set_bkg_tiles(x,y,2,3,redcard_color_map);
    else if(rank==0 && suit==HOLE) set_bkg_tiles(x,y,2,3,blackcard_color_map);
    else if(suit&2) set_bkg_tiles(x,y,2,3,redcard_color_map);
    else set_bkg_tiles(x,y,2,3,blackcard_color_map);
    VBK_REG = 0; 
  }
  set_bkg_tiles(x,y,2,3,card_maps[index]);
}

/***************************************************************
  Draws the top two tiles of a card on the screen
***************************************************************/
void draw_partial_card(UBYTE x,UBYTE y,UBYTE rank,UBYTE suit)
{
  UBYTE index;
  
  index=(rank<<2)+suit-1;

  if(_cpu == CGB_TYPE)
  { 
    VBK_REG = 1;
    if(rank==0 && suit==BACK) set_bkg_tiles(x,y,2,1,redcard_color_map);
    //else if(rank==0 && suit==HOLE) set_bkg_tiles(x,y,2,3,blackcard_color_map);
    else if(suit&2) set_bkg_tiles(x,y,2,1,redcard_color_map);
    else set_bkg_tiles(x,y,2,1,blackcard_color_map);
    VBK_REG = 0; 
  }
  set_bkg_tiles(x,y,2,1,card_maps[index]);
}

/***************************************************************
  Redraw all the cards on the screen
***************************************************************/
void refresh_cards(void)
{
  UBYTE i,j,position;
  
  //Draw stock
  if(deckindex==0) draw_card(0,0,0,HOLE);
  else draw_card(0,0,0,BACK);
  
  //Draw discard
  if(discardindex==0) draw_card(3,0,0,HOLE);
  else draw_card(3,0,get_rank(discard[discardindex-1]),get_suit(discard[discardindex-1]));
  
  //Draw goals
  for(i=0;i<4;i++)
  {
    if(goalindex[i]==0) draw_card(i*3+9,0,0,HOLE);
    else 
    {
      draw_card(i*3+9,0,get_rank(goal[i][goalindex[i]-1]),get_suit(goal[i][goalindex[i]-1]));
    }
  }
  
  //Draw foundations
  for(i=0;i<7;i++)
  {
    if(foundindex[i]==0)
    {
      draw_card(i*3,4,0,HOLE);
      if(_cpu == CGB_TYPE)
      { 
        VBK_REG = 1;
        set_bkg_tiles(i*3,7,2,11,background_color_map);
        VBK_REG = 0; 
      }
      set_bkg_tiles(i*3,7,2,11,background_tiles);
    }
    else 
    {
      if(foundindex[i]==1)  //only one card in pile
      {
        if(get_visibility(foundation[i][0])==FALSE) draw_card(i*3,4,0,BACK);
        else draw_card(i*3,4,get_rank(foundation[i][0]),get_suit(foundation[i][0]));
        if(_cpu == CGB_TYPE)
        { 
          VBK_REG = 1;
          set_bkg_tiles(i*3,7,2,11,background_color_map);
          VBK_REG = 0; 
        }
        set_bkg_tiles(i*3,7,2,11,background_tiles);
      }
      else //more than one card in pile
      {
        position=4;
        if(get_visibility(foundation[i][0])==FALSE)
        {
          draw_partial_card(i*3,position,0,BACK); //trapped cards
          position++;
        }
        for(j=0;j<foundindex[i];j++)
        {
          if(get_rank(foundation[i][j+1])==0 && get_visibility(foundation[i][j])==FALSE)
          {
            draw_card(i*3,position,0,BACK);
            position++;
          }
          else if(get_visibility(foundation[i][j])==TRUE)
          {
            if(j==foundindex[i]-1) draw_card(i*3,position,get_rank(foundation[i][j]),get_suit(foundation[i][j]));
            else draw_partial_card(i*3,position,get_rank(foundation[i][j]),get_suit(foundation[i][j]));
            position++;
          }
        }  
        if(_cpu == CGB_TYPE)
        { 
          VBK_REG = 1;
          set_bkg_tiles(i*3,position+2,2,11,background_color_map);
          VBK_REG = 0; 
        }
        set_bkg_tiles(i*3,position+2,2,11,background_tiles);
      }
    }
  }
}

/***************************************************************
  Moves the pointer
***************************************************************/
void place_pointer(UBYTE x, UBYTE y)
{
  move_sprite(0,x+8,y+16);
  move_sprite(1,x+8,y+16+8);
}

/***************************************************************
  Moves the dot
***************************************************************/
void place_dot(UBYTE x, UBYTE y)
{
  move_sprite(2,x+4,y+12);
}

/***************************************************************
  Determines what to do when the A button is pressed
***************************************************************/
void process_click(void)
{
  UBYTE i,temp,clicked_pile,active_pile,grabbed_card;
  
  waitpadup();
  
  if(pointer_x<=16 && pointer_y<=24) //click stock
  {
    if(deckindex!=0) //stock is not empty
    {
      for(i=0;i<num_to_draw;i++)
      {
        if(deckindex>0) move_deck_discard();
      }
    }
    else //stock is empty
    {
      while(discardindex!=0)
      {
        move_discard_deck();
      }
    }
    discard_active=0;
    foundation_active=0;
    place_dot(200,200);
  }

  else if(pointer_x>=24 && pointer_x<=40 && pointer_y<=24) //click discard
  {
    if(discardindex!=0) //discard is not empty
    {
      place_dot(pointer_x,pointer_y);
      discard_active=1;
    }
  }

  else if(pointer_y<=24) //click goal area
  {
    clicked_pile=4;
    if(pointer_x>=72 && pointer_x<=88) clicked_pile=0;
    else if(pointer_x>=96 && pointer_x<=112) clicked_pile=1;
    else if(pointer_x>=120 && pointer_x<=136) clicked_pile=2;
    else if(pointer_x>=144 && pointer_x<=160) clicked_pile=3;
    if(discard_active) //discard has the dot on it
    {
      if(get_rank(goal[clicked_pile][goalindex[clicked_pile]-1])==get_rank(discard[discardindex-1])-1 
        && get_suit(goal[clicked_pile][goalindex[clicked_pile]-1])==get_suit(discard[discardindex-1]))
        {
          move_discard_goal(clicked_pile);
        }
      else if(get_rank(discard[discardindex-1])==1 && goalindex[clicked_pile]==0)
      {
        move_discard_goal(clicked_pile);
      }
      discard_active=0;
      place_dot(200,200);
    }
    else if(foundation_active)
    {
      foundation_active--;
      active_pile=(int)(foundation_active/14);
      i=foundindex[active_pile]-1;
      if(get_rank(goal[clicked_pile][goalindex[clicked_pile]-1])==get_rank(foundation[active_pile][i])-1 
        && get_suit(goal[clicked_pile][goalindex[clicked_pile]-1])==get_suit(foundation[active_pile][i]))
        {
          move_found_goal(active_pile,clicked_pile);
        }
      else if(get_rank(foundation[active_pile][i])==1 && goalindex[clicked_pile]==0)
      {
        move_found_goal(active_pile,clicked_pile);
      }
      foundation_active=0;
      place_dot(200,200);
    }
  }
  
  else if(pointer_y>=32)  //clicked the foundation area
  {
    if(pointer_x<=20) clicked_pile=0;
    else if(pointer_x>=20 && pointer_x<=44) clicked_pile=1;
    else if(pointer_x>=44 && pointer_x<=68) clicked_pile=2;
    else if(pointer_x>=68 && pointer_x<=92) clicked_pile=3;
    else if(pointer_x>=92 && pointer_x<=116) clicked_pile=4;
    else if(pointer_x>=116 && pointer_x<=140) clicked_pile=5;
    else if(pointer_x>=140 && pointer_x<=160) clicked_pile=6;
    
      if(get_visibility(foundation[clicked_pile][foundindex[clicked_pile]-1])==FALSE && foundindex[clicked_pile]!=0)
      {
        foundation[clicked_pile][foundindex[clicked_pile]-1]=make_visible(foundation[clicked_pile][foundindex[clicked_pile]-1]);
        place_dot(200,200);
      }
      else if(discard_active) //discard has the dot on it
      {
        if(get_rank(foundation[clicked_pile][foundindex[clicked_pile]-1])==get_rank(discard[discardindex-1])+1 
          && (get_suit(foundation[clicked_pile][foundindex[clicked_pile]-1])&2)!=(get_suit(discard[discardindex-1])&2))
          {
            move_discard_found(clicked_pile);
          }
        else if(get_rank(discard[discardindex-1])==13 && foundindex[clicked_pile]==0)
        {
          move_discard_found(clicked_pile);
        }
        discard_active=0;
        place_dot(200,200);
      }
      else if(foundation_active) //another foundation has dot on it
      {
        foundation_active--;
        active_pile=(int)(foundation_active/14);
        grabbed_card=foundation_active-active_pile*14;
        if(get_visibility(foundation[active_pile][0])==FALSE)
        {
          i=1;
          while(get_visibility(foundation[active_pile][i])==FALSE)
          {
            grabbed_card++;
            i++;
          }
        }
        i=foundindex[active_pile]-1;        
        if(grabbed_card>i) grabbed_card=i;
        i=grabbed_card;
        
        if((get_rank(foundation[clicked_pile][foundindex[clicked_pile]-1]))==(get_rank(foundation[active_pile][i])+1) 
          && (get_suit(foundation[clicked_pile][foundindex[clicked_pile]-1])&2)!=(get_suit(foundation[active_pile][i])&2))
          {
            if(get_visibility(foundation[active_pile][i])==TRUE)
              move_found_found(foundindex[active_pile]-grabbed_card,active_pile,clicked_pile);
          }
        else if(get_rank(foundation[active_pile][i])==13 && foundindex[clicked_pile]==0)
        {
          if(get_visibility(foundation[active_pile][i])==TRUE)
            move_found_found(foundindex[active_pile]-grabbed_card,active_pile,clicked_pile);
        }
        foundation_active=0;
        place_dot(200,200);
      }
      else
      {
        foundation_active=(pointer_y>>3)-3+clicked_pile*14;
        place_dot(pointer_x,pointer_y);
      }
  }
  
  else
  {
    discard_active=0;
    foundation_active=0;
    place_dot(200,200);
  }

  refresh_cards();
}

/*******************************************************
  Makes a card visible
*******************************************************/
UBYTE make_visible(UBYTE card)
{
  card|=128;
  return card;
}

/*******************************************************
  Get the properties of a card
*******************************************************/
UBYTE get_suit(UBYTE card)
{
  return (card & SUIT_MASK);
}

UBYTE get_rank(UBYTE card)
{
  return ((card & RANK_MASK)>>2);
}

UBYTE get_visibility(UBYTE card)
{
  return ((card & VISIBILITY_MASK)>>7);
}

/*******************************************************
  Deal the cards
*******************************************************/
void deal_cards(void)
{
  UBYTE i,j,card;
  UBYTE usedcard[52];
    
  discard_active=0;
  foundation_active=0;

  for(i=0;i<52;i++) deck[i]=0;
  for(i=0;i<7;i++) for(j=0;j<19;j++) foundation[i][j]=0;
  for(i=0;i<24;i++) discard[i]=0;
  for(i=0;i<4;i++) for(j=0;j<13;j++) goal[i][j]=0;

  deckindex=0;
  discardindex=0;
  for(i=0;i<4;i++) goalindex[i]=0;
  for(i=0;i<7;i++) foundindex[i]=0;

  for(i=0;i<52;i++) usedcard[i]=FALSE;

  for(i=0;i<52;i++)
  {
    do
    {
      do{card=rand();}while(card>51);
    }while(card>51||usedcard[card]);
    usedcard[card]=TRUE;
    do
    {
      do{deckindex=rand();}while(deckindex>51);
    }while(get_rank(deck[deckindex])!=0);
    deck[deckindex]=card+4;  //sets rank, suit, and invisibility in one pop
  }
  
  deckindex=52;

  for(i=0;i<=6;i++)
  {
    for(j=i;j<=6;j++)
    {
      move_deck_found(j);
    }
  }
  for(i=0;i<7;i++)
    foundation[i][i]=make_visible(foundation[i][i]);
}

/*******************************************************
  Move a card from the deck to the discard
*******************************************************/
void move_deck_discard(void)
{
  deckindex--;
  discard[discardindex]=deck[deckindex];
  deck[deckindex]=0;
  discardindex++;
}

/*******************************************************
  Move a card from the discard to the stock
*******************************************************/
void move_discard_deck(void)
{
  discardindex--;
  deck[deckindex]=discard[discardindex];
  discard[discardindex]=0;
  deckindex++;
}

/*******************************************************
  Move a card from the deck to a foundation
*******************************************************/
void move_deck_found(UBYTE pile)
{
  deckindex--;
  foundation[pile][foundindex[pile]]=deck[deckindex];
  deck[deckindex]=0;
  foundindex[pile]++;
}

/*******************************************************
  Move a card from the discard to a goal
*******************************************************/
void move_discard_goal(UBYTE pile)
{
  discardindex--;
  goal[pile][goalindex[pile]]=discard[discardindex];
  discard[discardindex]=0;
  goalindex[pile]++;
}

/*******************************************************
  Move a card from the discard to a foundation
*******************************************************/
void move_discard_found(UBYTE pile)
{
  discardindex--;

  foundation[pile][foundindex[pile]]=discard[discardindex];
  foundation[pile][foundindex[pile]]=make_visible(foundation[pile][foundindex[pile]]);

  discard[discardindex]=0;
  foundindex[pile]++;
}

/*******************************************************
  Move a card from a foundation to a goal
*******************************************************/
void move_found_goal(UBYTE found_pile, UBYTE goal_pile)
{
  UBYTE i;
  
  foundindex[found_pile]--;

  goal[goal_pile][goalindex[goal_pile]]=foundation[found_pile][foundindex[found_pile]];

  foundation[found_pile][foundindex[found_pile]]=0;
  goalindex[goal_pile]++;

  /*if(_cpu == CGB_TYPE)
  { 
    VBK_REG = 1;
    set_bkg_tiles(found_pile*3,7,2,11,background_color_map);
    VBK_REG = 0; 
  }
  set_bkg_tiles(found_pile*3,7,2,11,background_tiles);*/
}

/*******************************************************
  Move a card(s) from one foundation to another
*******************************************************/
void move_found_found(UBYTE cards_to_move, UBYTE source_pile, UBYTE dest_pile)
{
  UBYTE i;

  for(i=1;i<=cards_to_move;i++)
  {
    foundation[dest_pile][foundindex[dest_pile]+cards_to_move-i]=foundation[source_pile][foundindex[source_pile]-i];
    foundation[source_pile][foundindex[source_pile]-i]=0;
  }
  
  foundindex[source_pile]-=cards_to_move;
  foundindex[dest_pile]+=cards_to_move;

  /*if(_cpu == CGB_TYPE)
  { 
    VBK_REG = 1;
    set_bkg_tiles(source_pile*3,7,2,11,background_color_map);
    VBK_REG = 0; 
  }
  set_bkg_tiles(source_pile*3,7,2,11,background_tiles);*/
}

/*******************************************************
  Check for a button press
*******************************************************/
void check_joypad(void)
{
  UBYTE i,button_temp,temp,temp2,clicked_pile;
  
  button_temp=joypad();
  if((button_temp & J_SELECT) && (button_temp & J_DOWN))
  {
    deal_cards();
    //draw_background();
    refresh_cards();
  }
  if((button_temp & J_UP) && pointer_y!=0)
  {
    pointer_y--;
    place_pointer(pointer_x,pointer_y);
  }
  if((button_temp & J_DOWN) && pointer_y!=142)
  {
    pointer_y++;
    place_pointer(pointer_x,pointer_y);
  }
  if((button_temp & J_LEFT) && pointer_x!=0)
  {
    pointer_x--;
    place_pointer(pointer_x,pointer_y);
  }
  if((button_temp & J_RIGHT) && pointer_x!=158)
  {
    pointer_x++;
    place_pointer(pointer_x,pointer_y);
  }
  else if(button_temp & J_A) process_click();
  else if(button_temp & J_B)
  {
    waitpadup();

    place_dot(200,200);
    discard_active=0;
    foundation_active=0;
    
    if(pointer_x>=24 && pointer_x<=40 && pointer_y<=24) //click discard
    {
      for(i=0;i<4;i++)
      {
        temp=goalindex[i]-1;
        if(get_rank(goal[i][temp])==get_rank(discard[discardindex-1])-1 
          && get_suit(goal[i][temp])==get_suit(discard[discardindex-1]))
          {
            move_discard_goal(i);
            refresh_cards();
            i=4;
          }
        else if(get_rank(discard[discardindex-1])==1 && goalindex[i]==0)
        {
          move_discard_goal(i);
          refresh_cards();
          i=4;
        }
      }
    }
    else if(pointer_y>=32)  //clicked the foundation area
    {
      if(/*pointer_x>=0 && */pointer_x<=20) clicked_pile=0;
      else if(pointer_x>=20 && pointer_x<=44) clicked_pile=1;
      else if(pointer_x>=44 && pointer_x<=68) clicked_pile=2;
      else if(pointer_x>=68 && pointer_x<=92) clicked_pile=3;
      else if(pointer_x>=92 && pointer_x<=116) clicked_pile=4;
      else if(pointer_x>=116 && pointer_x<=140) clicked_pile=5;
      else if(pointer_x>=140 && pointer_x<=160) clicked_pile=6;
    
      temp=foundindex[clicked_pile]-1;
      if(get_visibility(foundation[clicked_pile][temp])==FALSE)
      {
        //don't do anything
      }
      else if(clicked_pile!=7)
      {
        for(i=0;i<4;i++)
        {
          temp2=goalindex[i]-1;
          if(get_rank(goal[i][temp2])==get_rank(foundation[clicked_pile][temp])-1 
            && get_suit(goal[i][temp2])==get_suit(foundation[clicked_pile][temp]))
            {
              move_found_goal(clicked_pile,i);
              refresh_cards();
              i=4;
            }
          else if(get_rank(foundation[clicked_pile][temp])==1 && goalindex[i]==0)
          {
            move_found_goal(clicked_pile,i);
            refresh_cards();
            i=4;
          }
        }
      }
    }
  }
  else if(button_temp & J_START) pause_game();
}

/***************************************************************
  Pause routine
***************************************************************/
void pause_game(void)
{
  waitpadup();
  settings_screen();
}
/***************************************************************
  Throws some card around when you win
***************************************************************/
void win_animation(void)
{
  UBYTE x,y,card;
  
  waitpadup();
  while(!(joypad()))
  {
    do{card=rand();}while(card>51);
    do{x=rand();}while(x>20);
    do{y=rand();}while(y>18);
    if(_cpu == CGB_TYPE)
    { 
      VBK_REG = 1;
      if(card&2) set_bkg_tiles(x,y,2,3,redcard_color_map);
      else set_bkg_tiles(x,y,2,3,blackcard_color_map);
      VBK_REG = 0; 
    }
    set_bkg_tiles(x,y,2,3,card_maps[card+3]);
  }
  deal_cards();
  draw_background();
  refresh_cards();
}

/***************************************************************
  Main program
***************************************************************/
void main()
{
  num_to_draw=3;

  init_graphics();
  title_screen();
  draw_background();
  settings_screen();
  deal_cards();
  refresh_cards();
  place_pointer(pointer_x,pointer_y);
  while(1)
  {
    delay(8);
    check_joypad();
    if(get_rank(goal[0][12])==13 &&
       get_rank(goal[1][12])==13 &&
       get_rank(goal[2][12])==13 &&
       get_rank(goal[3][12])==13) win_animation();
  }
}
